@@ -1,8 +1,12 @@
-# django imports
+# python imports
+import re
+
+# # django imports
 from django.contrib.contenttypes import generic
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.cache import cache
+from django.core.urlresolvers import reverse
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
 
@@ -10,16 +14,17 @@ from django.utils.translation import ugettext_lazy as _
 import lfs.catalog.utils
 from lfs.core.fields.thumbs import ImageWithThumbsField
 from lfs.core.managers import ActiveManager
-from lfs.catalog.settings import ACTIVE_FOR_SALE_CHOICES
+from lfs.catalog.settings import ACTIVE_FOR_SALE_CHOICES, CONTENT_CATEGORIES
 from lfs.catalog.settings import ACTIVE_FOR_SALE_STANDARD
 from lfs.catalog.settings import ACTIVE_FOR_SALE_YES
 from lfs.catalog.settings import PRODUCT_TYPE_CHOICES
+from lfs.catalog.settings import CONFIGURABLE_PRODUCT
 from lfs.catalog.settings import STANDARD_PRODUCT
 from lfs.catalog.settings import VARIANT
 from lfs.catalog.settings import PRODUCT_WITH_VARIANTS
-from lfs.catalog.settings import VARIANTS_DISPLAY_TYPE_CHOICES
+from lfs.catalog.settings import CAT_CATEGORY_PATH
+from lfs.catalog.settings import CATEGORY_TEMPLATES
 from lfs.catalog.settings import CONTENT_PRODUCTS
-from lfs.catalog.settings import CONTENT_CHOICES
 from lfs.catalog.settings import LIST
 from lfs.catalog.settings import DELIVERY_TIME_UNIT_CHOICES
 from lfs.catalog.settings import DELIVERY_TIME_UNIT_SINGULAR
@@ -27,15 +32,21 @@ from lfs.catalog.settings import DELIVERY_TIME_UNIT_HOURS
 from lfs.catalog.settings import DELIVERY_TIME_UNIT_DAYS
 from lfs.catalog.settings import DELIVERY_TIME_UNIT_WEEKS
 from lfs.catalog.settings import DELIVERY_TIME_UNIT_MONTHS
-from lfs.catalog.settings import PROPERTY_TYPE_CHOICES
-from lfs.catalog.settings import PROPERTY_TEXT_FIELD
-from lfs.catalog.settings import PROPERTY_SELECT_FIELD
+from lfs.catalog.settings import PROPERTY_FIELD_CHOICES
 from lfs.catalog.settings import PROPERTY_NUMBER_FIELD
+from lfs.catalog.settings import PROPERTY_SELECT_FIELD
+from lfs.catalog.settings import PROPERTY_TEXT_FIELD
 from lfs.catalog.settings import PROPERTY_STEP_TYPE_CHOICES
 from lfs.catalog.settings import PROPERTY_STEP_TYPE_AUTOMATIC
 from lfs.catalog.settings import PROPERTY_STEP_TYPE_MANUAL_STEPS
 from lfs.catalog.settings import PROPERTY_STEP_TYPE_FIXED_STEP
+from lfs.catalog.settings import PROPERTY_VALUE_TYPE_DEFAULT
+from lfs.catalog.settings import PROPERTY_VALUE_TYPE_DISPLAY
+from lfs.catalog.settings import PROPERTY_VALUE_TYPE_VARIANT
+from lfs.catalog.settings import PRODUCT_TEMPLATES
+from lfs.catalog.settings import VARIANTS_DISPLAY_TYPE_CHOICES
 from lfs.tax.models import Tax
+from lfs.manufacturer.models import Manufacturer
 
 class Category(models.Model):
     """A category is used to browse through the shop products. A category can
@@ -78,10 +89,6 @@ class Category(models.Model):
         - static_block
             A assigned static block to the category.
 
-        - content
-            decides which content will be displayed. At the moment this is either
-            sub categories or products.
-
         - active_formats
             If True product_rows, product_cols and category_cols are taken from
             the category otherwise from the parent.
@@ -104,6 +111,10 @@ class Category(models.Model):
         - level
            The level of the category within the category hierachie, e.g. if it
            is a top level category the level is 1.
+
+        - template
+           Sets the template which renders the category view. If left to None, default template is used.
+
     """
     name = models.CharField(_(u"Name"), max_length=50)
     slug = models.SlugField(_(u"Slug"),unique=True)
@@ -121,7 +132,7 @@ class Category(models.Model):
     exclude_from_navigation = models.BooleanField(_(u"Exclude from navigation"), default=False)
 
     static_block = models.ForeignKey("StaticBlock", verbose_name=_(u"Static block"), blank=True, null=True, related_name="categories")
-    content = models.IntegerField(_(u"Content"), default=CONTENT_PRODUCTS, choices=CONTENT_CHOICES)
+    template = models.PositiveSmallIntegerField(_(u"Category template"), max_length=400, blank=True, null=True, choices=CATEGORY_TEMPLATES)
     active_formats = models.BooleanField(_(u"Active formats"), default=False)
 
     product_rows  = models.IntegerField(_(u"Product rows"), default=3)
@@ -296,7 +307,7 @@ class Category(models.Model):
 
         products = lfs.catalog.models.Product.objects.distinct().filter(
             active=True,
-            categories__in = categories).exclude(sub_type=VARIANT)
+            categories__in = categories).exclude(sub_type=VARIANT).distinct()
 
         cache.set(cache_key, products)
         return products
@@ -329,6 +340,26 @@ class Category(models.Model):
         import lfs.core.utils
         return self.parent or lfs.core.utils.get_default_shop()
 
+
+    def get_template_name(self):
+        """method to return the path of the category template
+        """
+        if self.template != None:
+            id = int(self.template)
+            return CATEGORY_TEMPLATES[id][1]["file"]
+
+        return None
+
+    def get_content(self):
+        """try to find out which type of content the template is rendering,
+        depending on its path.
+        """
+        if self.get_template_name() == None:
+           return CONTENT_PRODUCTS
+        if self.get_template_name().startswith(CAT_CATEGORY_PATH): # do we have category - templates
+           return CONTENT_CATEGORIES
+        return CONTENT_PRODUCTS
+
 class Product(models.Model):
     """A product is sold within a shop.
 
@@ -348,6 +379,13 @@ class Product(models.Model):
         - effective_price:
             Only for internal usage (price filtering).
 
+        - unit
+            The unit of the product. This is displayed beside the quantity
+            field.
+
+        - price_unit
+            The unit of the product's price. This is displayed beside the price
+
         - short_description
             The short description of the product. This is used within overviews.
 
@@ -418,6 +456,9 @@ class Product(models.Model):
         - tax
             Tax rate of the product.
 
+        - static_block
+            A static block which has been assigned to the product.
+
         - sub_type
             Sub type of the product. At the moment that is standard, product with
             variants, variant.
@@ -437,6 +478,21 @@ class Product(models.Model):
         - active_xxx
             If set to true the information will be taken from the variant.
             Otherwise from the parent product (only relevant for variants)
+
+        - template
+            Sets the template, which renders the product content. If left to None, default template is used.
+
+        - active_price_calculation
+            If True the price will be calculated by the field price_calculation
+
+        - price_calculation
+            Formula to calculate price of the product.
+
+        - sku_manufacturer
+            The product's article ID of the manufacturer (external article id)
+
+        - manufacturer
+            The manufacturer of the product.
     """
     # All products
     name = models.CharField(_(u"Name"), max_length=80, blank=True)
@@ -444,6 +500,8 @@ class Product(models.Model):
     sku = models.CharField(_(u"SKU"), blank=True, max_length=30)
     price = models.FloatField(_(u"Price"), default=0.0)
     effective_price = models.FloatField(_(u"Price"), blank=True)
+    price_unit = models.CharField(blank=True, max_length=20)
+    unit = models.CharField(blank=True, max_length=20)
     short_description = models.TextField(_(u"Short description"), blank=True)
     description = models.TextField(_(u"Description"), blank=True)
     images = generic.GenericRelation("Image", verbose_name=_(u"Images"),
@@ -471,9 +529,15 @@ class Product(models.Model):
     delivery_time = models.ForeignKey("DeliveryTime", verbose_name=_(u"Delivery time"), blank=True, null=True, related_name="products_delivery_time")
     order_time = models.ForeignKey("DeliveryTime", verbose_name=_(u"Order time"), blank=True, null=True, related_name="products_order_time")
     ordered_at = models.DateField(_(u"Ordered at"), blank=True, null=True)
-    manage_stock_amount = models.BooleanField(_(u"Manage stock amount"), default=True)
+    manage_stock_amount = models.BooleanField(_(u"Manage stock amount"), default=False)
     stock_amount = models.FloatField(_(u"Stock amount"), default=0)
 
+    active_packing_unit = models.BooleanField(_(u"Active packing unit"), default=False)
+    packing_unit = models.FloatField(_(u"Packing unit"), blank=True, null=True)
+    packing_unit_unit = models.CharField(_(u"Unit"), blank=True, max_length=30)
+
+    static_block = models.ForeignKey("StaticBlock", verbose_name=_(u"Static block"), blank=True, null=True, related_name="products")
+
     # Dimension
     weight = models.FloatField(_(u"Weight"), default=0.0)
     height = models.FloatField(_(u"Height"), default=0.0)
@@ -496,6 +560,7 @@ class Product(models.Model):
     active_name = models.BooleanField(_(u"Active name"), default=False)
     active_sku = models.BooleanField(_(u"Active SKU"), default=False)
     active_short_description = models.BooleanField(_(u"Active short description"), default=False)
+    active_static_block = models.BooleanField(_(u"Active static bock"), default=False)
     active_description = models.BooleanField(_(u"Active description"), default=False)
     active_price = models.BooleanField(_(u"Active price"), default=False)
     active_for_sale = models.PositiveSmallIntegerField(_("Active for sale"), choices=ACTIVE_FOR_SALE_CHOICES, default=ACTIVE_FOR_SALE_STANDARD)
@@ -507,6 +572,15 @@ class Product(models.Model):
     active_meta_description = models.BooleanField(_(u"Active meta description"), default=False)
     active_meta_keywords = models.BooleanField(_(u"Active meta keywords"), default=False)
     active_dimensions = models.BooleanField(_(u"Active dimensions"), default=False)
+    template = models.PositiveSmallIntegerField(_(u"Product template"), blank=True, null=True, max_length=400, choices=PRODUCT_TEMPLATES)
+
+    # Price calculation
+    active_price_calculation = models.BooleanField(_(u"Active price calculation"), default=False)
+    price_calculation = models.CharField(_(u"Price Calculation"), blank=True, max_length=100)
+
+    # Manufacturer
+    sku_manufacturer = models.CharField(blank=True, max_length=100)
+    manufacturer = models.ForeignKey(Manufacturer, blank=True, null=True, related_name="products")
 
     objects = ActiveManager()
 
@@ -750,27 +824,68 @@ class Product(models.Model):
         except KeyError:
             return None
 
-    def get_options(self):
-        """Returns the property value of a Variant in the correct
-        ordering of the properties.
+    def get_displayed_properties(self):
+        """Returns properties with ``display_on_product`` is True.
         """
-        cache_key = "product-property-values-%s" % self.id
-        options = cache.get(cache_key)
-        if options is None:
-            temp = []
-            for property_value in self.property_values.all():
-                temp.append((property_value, property_value.property.position))
+        cache_key = "displayed-properties-%s" % self.id
 
-            # TODO: Optimize
-            temp.sort(lambda a,b: cmp(a[1], b[1]))
+        properties = cache.get(cache_key)
+        if properties:
+            return properties
 
-            options = []
-            for option in temp:
-                options.append(option[0])
+        properties = []
+        for ppv in self.property_values.filter(property__display_on_product=True, type=PROPERTY_VALUE_TYPE_DISPLAY).order_by("property__position"):
+            if ppv.property.is_select_field:
+                try:
+                    po = PropertyOption.objects.get(pk=int(float(ppv.value)))
+                except (PropertyOption.DoesNotExist, ValueError):
+                    continue
+                else:
+                    value = po.name
+            else:
+                value = ppv.value
+            properties.append({
+                "name"  : ppv.property.name,
+                "title" : ppv.property.title,
+                "value" : value,
+                "unit"  : ppv.property.unit,
+            })
 
-            cache.set(cache_key, options)
+        cache.set(cache_key, properties)
+        return properties
 
-        return options
+    def get_variant_properties(self):
+        """Returns the property value of a variant in the correct ordering
+        of the properties.
+        """
+        cache_key = "variant-properties-%s" % self.id
+
+        properties = cache.get(cache_key)
+        if properties:
+            return properties
+
+        properties = []
+
+        for ppv in self.property_values.filter(type=PROPERTY_VALUE_TYPE_VARIANT).order_by("property__position"):
+            if ppv.property.is_select_field:
+                try:
+                    po = PropertyOption.objects.get(pk=int(float(ppv.value)))
+                except PropertyOption.DoesNotExist:
+                    continue
+                else:
+                    value = po.name
+            else:
+                value = ppv.value
+            properties.append({
+                "name"  : ppv.property.name,
+                "title" : ppv.property.title,
+                "value" : value,
+                "unit"  : ppv.property.unit,
+            })
+
+        cache.set(cache_key, properties)
+
+        return properties
 
     def has_option(self, property, option):
         """Returns True if the variant has the given property / option
@@ -788,14 +903,39 @@ class Product(models.Model):
         except KeyError:
             return False
 
-    def get_price(self):
+    def _get_default_properties_price(self, object):
+        """Returns the total price of all default properties.
+        """
+        price = 0
+        for property in object.get_configurable_properties():
+            if property.add_price:
+                # Try to get the default value of the property
+                try:
+                    ppv = ProductPropertyValue.objects.get(product=self, property=property, type=PROPERTY_VALUE_TYPE_DEFAULT)
+                    po = PropertyOption.objects.get(pk = ppv.value)
+                except (ObjectDoesNotExist, ValueError):
+                    # If there is no explicit default value try to get the first
+                    # option.
+                    if property.required:
+                        try:
+                            po = property.options.all()[0]
+                        except IndexError:
+                            continue
+                        else:
+                            price += po.price
+                else:
+                    price += po.price
+
+        return price
+
+    def get_price(self, with_properties=True):
         """Returns the price of the product. At the moment this is just the
         gross price. Later this could be the net or the gross price dependent on
         selected shop options.
         """
-        return self.get_price_gross()
+        return self.get_price_gross(with_properties)
 
-    def get_standard_price(self):
+    def get_standard_price(self, with_properties=True):
         """Returns always the standard price for the product. Independent
         whether the product is for sale or not. If you want the real price of
         the product use get_price instead.
@@ -808,9 +948,13 @@ class Product(models.Model):
         if object.is_variant() and not object.active_price:
             object = object.parent
 
-        return object.price
+        price = object.price
+        if with_properties and object.is_configurable_product():
+            price += self._get_default_properties_price(object)
 
-    def get_for_sale_price(self):
+        return price
+
+    def get_for_sale_price(self, with_properties=True):
         """returns the sale price for the product.
         """
         object = self
@@ -821,10 +965,21 @@ class Product(models.Model):
         if object.is_variant() and not object.active_for_sale_price:
             object = object.parent
 
-        return object.for_sale_price
+        price = object.for_sale_price
+        if with_properties and object.is_configurable_product():
+            price += self._get_default_properties_price(object)
+
+        return price
+
+    def get_price_gross(self, with_properties=True):
+        """Returns the real gross price of the product. This is the base of
+        all price and tax calculations.
+
+        **Parameters:**
+
+        with_properties
+            If True the prices of the default property options are added.
 
-    def get_price_gross(self):
-        """Returns the real gross price of the product.
         """
         object = self
 
@@ -833,14 +988,48 @@ class Product(models.Model):
 
         if object.get_for_sale():
             if object.is_variant() and not object.active_for_sale_price:
-                return object.parent.get_for_sale_price()
+                price = object.parent.get_for_sale_price()
             else:
-                return object.get_for_sale_price()
+                price = object.get_for_sale_price()
         else:
             if object.is_variant() and not object.active_price:
-                return object.parent.price
+                price = object.parent.price
             else:
-                return object.price
+                price = object.price
+
+        if with_properties and object.is_configurable_product():
+            price += self._get_default_properties_price(object)
+
+        return price
+
+    def get_price_with_unit(self):
+        """Returns the formatted gross price of the product
+        """
+        from lfs.core.templatetags.lfs_tags import currency
+        price = currency(self.get_price())
+
+        if self.price_unit:
+            price += " / " + self.price_unit
+
+        return price
+
+    def calculate_price(self, price):
+        """Calulates the price by given entered price calculation.
+        """
+        pc = self.price_calculation
+        tokens = self.price_calculation.split(" ")
+
+        for token in tokens:
+            if token.startswith("property"):
+                mo = re.match("property\((\d+)\)")
+                ppv = ProductPropertyValue.objects.get(product=self, property_id=mo.groups()[0])
+
+        try:
+            mult = float(self.price_calculation)
+        except:
+            mult = 1
+
+        return mult * price
 
     def get_price_net(self):
         """Returns the real net price of the product. Takes care whether the
@@ -868,6 +1057,36 @@ class Product(models.Model):
         properties = self.get_global_properties()
         properties.extend(self.get_local_properties())
 
+        properties.sort(lambda a, b: cmp(a.position, b.position))
+
+        return properties
+
+    def get_property_select_fields(self):
+        """Returns all properties which are `select types`.
+        """
+        # global
+        properties = []
+        for property_group in self.property_groups.all():
+            properties.extend(property_group.properties.filter(type=PROPERTY_SELECT_FIELD).order_by("groupspropertiesrelation"))
+
+        # local
+        for property in self.properties.filter(type=PROPERTY_SELECT_FIELD).order_by("productspropertiesrelation"):
+            properties.append(property)
+
+        return properties
+
+    def get_configurable_properties(self):
+        """Returns all properties which are configurable.
+        """
+        # global
+        properties = []
+        for property_group in self.property_groups.all():
+            properties.extend(property_group.properties.filter(configurable=True).order_by("groupspropertiesrelation"))
+
+        # local
+        for property in self.properties.filter(configurable=True).order_by("productspropertiesrelation"):
+            properties.append(property)
+
         return properties
 
     def get_sku(self):
@@ -911,13 +1130,10 @@ class Product(models.Model):
         related_products = cache.get(cache_key)
 
         if related_products is None:
-
             if self.is_variant() and not self.active_related_products:
-                related_products = self.parent.related_products.exclude(
-                    sub_type=PRODUCT_WITH_VARIANTS)
+                related_products = self.parent.related_products.all()
             else:
-                related_products = self.related_products.exclude(
-                    sub_type=PRODUCT_WITH_VARIANTS)
+                related_products = self.related_products.all()
 
             cache.set(cache_key, related_products)
 
@@ -929,14 +1145,41 @@ class Product(models.Model):
         This is either a selected variant or the first added variant. If the
         product has no variants it is None.
         """
+        cache_key = "default-variant-%s" % self.id
+        default_variant = cache.get(cache_key)
+
+        if default_variant is not None:
+            return default_variant
+
         if self.default_variant is not None:
-            return self.default_variant
+            default_variant = self.default_variant
         else:
             try:
-                return self.variants.filter(active=True)[0]
+                default_variant = self.variants.filter(active=True)[0]
             except IndexError:
                 return None
 
+        cache.set(cache_key, default_variant)
+        return default_variant
+
+    def get_static_block(self):
+        """Returns the static block of the product. Takes care whether the
+        product is a variant and meta description are active or not.
+        """
+        cache_key = "product-static-block-%s" % self.id
+        block = cache.get(cache_key)
+        if block is not None:
+            return block
+
+        if self.is_variant() and not self.active_static_block:
+            block = self.parent.static_block
+        else:
+            block = self.static_block
+
+        cache.set(cache_key, block)
+
+        return block
+
     def get_variants(self):
         """Returns the variants of the product.
         """
@@ -1018,11 +1261,26 @@ class Product(models.Model):
         else:
             return self.height
 
+    def get_packing_info(self):
+        """Returns the packing info of the product as list:
+        """
+        if self.is_variant():
+            obj = self.parent
+        else:
+            obj = self
+
+        return (obj.packing_unit, obj.packing_unit_unit)
+
     def is_standard(self):
         """Returns True if product is standard product.
         """
         return self.sub_type == STANDARD_PRODUCT
 
+    def is_configurable_product(self):
+        """Returns True if product is configurable product.
+        """
+        return self.sub_type == CONFIGURABLE_PRODUCT
+
     def is_product_with_variants(self):
         """Returns True if product is product with variants.
         """
@@ -1044,9 +1302,9 @@ class Product(models.Model):
     def is_deliverable(self):
         """Returns the deliverable state of the product.
         """
-        if self.is_variant():
-            return self.parent.deliverable
-        else:
+        if self.manage_stock_amount and self.stock_amount <= 0 and not self.order_time:
+            return False
+        else:    
             return self.deliverable
 
     # 3rd party contracts
@@ -1062,7 +1320,14 @@ class Product(models.Model):
                 return self.categories.all()[0]
             except:
                 return None
-
+    def get_template_name(self):
+        """
+        method to return the path of the product template
+        """
+        if self.template != None:
+            id = int(self.template)
+            return PRODUCT_TEMPLATES[id][1]["file"]
+        return None
 
 class ProductAccessories(models.Model):
     """Represents the relationship between products and accessories.
@@ -1084,7 +1349,7 @@ class ProductAccessories(models.Model):
           The proposed amount of accessories for the product.
     """
     product = models.ForeignKey("Product", verbose_name=_(u"Product"), related_name="productaccessories_product")
-    accessory = models.ForeignKey("Product", verbose_name=_(u"Acessory"), related_name="productaccessories_accessory")
+    accessory = models.ForeignKey("Product", verbose_name=_(u"Accessory"), related_name="productaccessories_accessory")
     position = models.IntegerField( _(u"Position"), default=999)
     quantity = models.FloatField(_(u"Quantity"), default=1)
 
@@ -1106,18 +1371,33 @@ class PropertyGroup(models.Model):
 
     Can belong to several products, products can have several groups
 
-    Attributes:
-        - name
-          The name of the property group.
-        - products
+    **Attributes**:
+
+    name
+        The name of the property group.
+
+    products
           The assigned products of the property group.
     """
     name = models.CharField(blank=True, max_length=50)
     products = models.ManyToManyField(Product, verbose_name=_(u"Products"), related_name="property_groups")
 
+    class Meta:
+        ordering = ("name", )
+
     def __unicode__(self):
         return self.name
 
+    def get_configurable_properties(self):
+        """Returns all configurable properties of the property group.
+        """
+        return self.properties.filter(configurable=True)
+
+    def get_filterable_properties(self):
+        """Returns all filterable properties of the property group.
+        """
+        return self.properties.filter(filterable=True)
+
 class Property(models.Model):
     """Represents a property of a product like color or size.
 
@@ -1126,29 +1406,74 @@ class Property(models.Model):
 
     A property belongs to exactly one group xor product.
 
-    Parameters:
-        - groups, product:
-            The group or product it belongs to. A property can belong to several
-            groups and/or to one product.
-        - name:
-            Is displayed within forms.
-        - position:
-            The position of the property within a product.
-        - filterable:
-            If True the property is used for filtered navigation.
-        - display_no_results
-            If True filter ranges with no products will be displayed. Otherwise
-            they will be removed.
-        - unit:
-            Something like cm, mm, m, etc.
-        - local
-            If True the property belongs to exactly one product
-        - type
-           char field, number field or select field
-        - step
-           manuel step for filtering
+    **Parameters**:
+
+    groups, product:
+        The group or product it belongs to. A property can belong to several
+        groups and/or to one product.
+
+    name:
+        Internal name of the property.
+
+    title:
+        Displayed to the customer.
+
+    position:
+        The position of the property within a product.
+
+    filterable:
+        If True the property is used for filtered navigation.
+
+    configurable
+        if True the property is used for configurable product.
+
+    display_no_results
+        If True filter ranges with no products will be displayed. Otherwise
+        they will be removed.
+
+    display_on_product
+        If True the property is displayed as an attribute on the product.
+
+    unit:
+        Something like cm, mm, m, etc.
+
+    local
+        If True the property belongs to exactly one product
+
+    type
+       char field, number field or select field
+
+    step
+       manuel step for filtering
+
+    price
+        The price of the property. Only used for configurable products.
+
+    unit_min
+        The minimal unit of the property the shop customer can enter.
+
+    unit_max
+        The maximal unit of the property the shop customer can enter.
+
+    unit_step
+        The step width the shop customer can edit.
+
+    decimal_places
+        The decimal places of a number field.
+
+    required
+        If True the field is required (for configurable properties).
+
+    display_price
+        If True the option price is displayed (for select field)
+
+    add_price
+        if True the option price is added to the product price (for select
+        field)
+
     """
     name = models.CharField( _(u"Name"), max_length=100)
+    title = models.CharField( _(u"Title"), max_length=100)
     groups = models.ManyToManyField(PropertyGroup, verbose_name=_(u"Group"), blank=True, null=True, through="GroupsPropertiesRelation", related_name="properties")
     products = models.ManyToManyField(Product, verbose_name=_(u"Products"), blank=True, null=True, through="ProductsPropertiesRelation", related_name="properties")
     position = models.IntegerField(_(u"Position"), blank=True, null=True)
@@ -1157,7 +1482,19 @@ class Property(models.Model):
     local = models.BooleanField(default=False)
     filterable = models.BooleanField(default=True)
     display_no_results = models.BooleanField(_(u"Display no results"), default=False)
-    type = models.PositiveSmallIntegerField(_(u"Type"), choices=PROPERTY_TYPE_CHOICES, default=PROPERTY_TEXT_FIELD)
+    configurable = models.BooleanField(default=False)
+    type = models.PositiveSmallIntegerField(_(u"Type"), choices=PROPERTY_FIELD_CHOICES, default=PROPERTY_TEXT_FIELD)
+    price = models.FloatField(_(u"Price"), blank=True, null=True)
+    display_price = models.BooleanField(_(u"Display price"), default=True)
+    add_price = models.BooleanField(_(u"Add price"), default=True)
+
+    # Number input field
+    unit_min = models.FloatField(_(u"Min"), blank=True, null=True)
+    unit_max = models.FloatField(_(u"Max"), blank=True, null=True)
+    unit_step = models.FloatField(_(u"Step"), blank=True, null=True)
+    decimal_places = models.PositiveSmallIntegerField(_(u"Decimal places"), default=0)
+
+    required = models.BooleanField(default=False)
 
     step_type = models.PositiveSmallIntegerField(_(u"Step type"), choices=PROPERTY_STEP_TYPE_CHOICES, default=PROPERTY_STEP_TYPE_AUTOMATIC)
     step = models.IntegerField(_(u"Step"), blank=True, null=True)
@@ -1166,7 +1503,7 @@ class Property(models.Model):
 
     class Meta:
         verbose_name_plural = _(u"Properties")
-        ordering = ["position"]
+        ordering = ["name"]
 
     def __unicode__(self):
         return self.name
@@ -1303,27 +1640,36 @@ class ProductPropertyValue(models.Model):
     """Stores the value resp. selected option of a product/property combination.
     This is some kind of EAV.
 
-    Attributes:
-        - product
-          The product for which the value is stored.
-        - parent_id
-          If the product is an variant this stores the parent id of it, if the
-          product is no variant it stores the id of the product itself. This is
-          just used to calculate the filters properly.
-        - property
-          The property for which the value is stored.
-        - value
-          The value for the product/property pair. Dependent of the property
-          type the value is either a number, a text or an id of an option.
+    *Attributes*:
+
+    product
+        The product for which the value is stored.
+
+    parent_id
+        If the product is an variant this stores the parent id of it, if the
+        product is no variant it stores the id of the product itself. This is
+        just used to calculate the filters properly.
+
+    property
+        The property for which the value is stored.
+
+    value
+        The value for the product/property pair. Dependent of the property
+        type the value is either a number, a text or an id of an option.
+
+    type
+        The type of the product value, which is on of "filter value",
+        "default value", "display value", "variant value"
     """
     product = models.ForeignKey(Product, verbose_name=_(u"Product"), related_name="property_values")
-    parent_id = models.IntegerField(blank=True, null=True)
+    parent_id = models.IntegerField(_(u"Parent"), blank=True, null=True)
     property = models.ForeignKey("Property", verbose_name=_(u"Property"), related_name="property_values")
-    value = models.CharField(blank=True, max_length=100)
-    value_as_float = models.FloatField(blank=True, null=True)
+    value = models.CharField(_(u"Value"), blank=True, max_length=100)
+    value_as_float = models.FloatField(_(u"Value as float"), blank=True, null=True)
+    type = models.PositiveSmallIntegerField(_(u"Type"))
 
     class Meta:
-        unique_together = ("product", "property", "value")
+        unique_together = ("product", "property", "value", "type")
 
     def __unicode__(self):
         return "%s/%s: %s" % (self.product.name, self.property.name, self.value)
@@ -1365,7 +1711,7 @@ class Image(models.Model):
     content = generic.GenericForeignKey(ct_field="content_type", fk_field="content_id")
 
     title = models.CharField(_(u"Title"), blank=True, max_length=100)
-    image = ImageWithThumbsField(_(u"Image"), upload_to="images", blank=True, null=True, sizes=((60, 60), (100, 100), (200, 200), (400, 400)))
+    image = ImageWithThumbsField(_(u"Image"), upload_to="images", blank=True, null=True, sizes=((60, 60), (100, 100), (200, 200), (300, 300), (400, 400)))
     position = models.PositiveSmallIntegerField(_(u"Position"), default=999)
 
     class Meta:
@@ -1374,17 +1720,72 @@ class Image(models.Model):
     def __unicode__(self):
         return self.title
 
+class File(models.Model):
+    """A downloadable file.
+
+    **Attributes:**
+
+    title
+        The title of the image. Used within the title tag of the file.
+
+    slug
+        The URL of the file.
+
+    content
+        The content object the file belongs to (optional).
+
+    position
+        The ordinal number within the content object. Used to order the files.
+
+    description
+        A long description of the file. Can be used within the content
+        (optional).
+
+    file
+        The binary file.
+    """
+    title = models.CharField(blank=True, max_length=100)
+    slug = models.SlugField()
+
+    content_type = models.ForeignKey(ContentType, verbose_name=_(u"Content type"), related_name="files", blank=True, null=True)
+    content_id = models.PositiveIntegerField(_(u"Content id"), blank=True, null=True)
+    content = generic.GenericForeignKey(ct_field="content_type", fk_field="content_id")
+
+    position = models.SmallIntegerField(default=999)
+    description = models.CharField(blank=True, max_length=100)
+    file = models.FileField(upload_to="files")
+
+    class Meta:
+        ordering = ("position", )
+
+    def __unicode__(self):
+        return self.title
+
+    def get_absolute_url(self):
+        return reverse("lfs_file", kwargs={"id" : self.id})
+
 class StaticBlock(models.Model):
     """A block of static HTML which can be assigned to content objects.
 
-    Attributes:
-        - name
-          The name of the static block.
-        - html
-          The static HTML of the block.
+    **Attributes**:
+
+    name
+        The name of the static block.
+
+    html
+        The static HTML of the block.
+
+    display_files
+        If True the files are displayed for download within the static block.
+
+    files
+        The files of the static block.
     """
     name = models.CharField(_(u"Name"), max_length=30)
-    html = models.TextField( _(u"HTML"), blank=True)
+    display_files = models.BooleanField(_(u"Display files"), default=True)
+    html = models.TextField(_(u"HTML"), blank=True)
+    files = generic.GenericRelation(File, verbose_name=_(u"Files"),
+        object_id_field="content_id", content_type_field="content_type")
 
     def __unicode__(self):
         return self.name
@@ -1413,6 +1814,11 @@ class DeliveryTime(models.Model):
     def __unicode__(self):
         return self.round().as_string()
 
+    def __gt__(self, other):
+        if self.max > other.max:
+            return True
+        return False
+
     def __add__(self, other):
         """Adds to delivery times.
         """
