@@ -4,11 +4,13 @@ import sys
 import urllib
 
 # django imports
+from django.conf import settings
 from django.http import HttpResponseRedirect
 from django.http import HttpResponse
 from django.utils import simplejson
 from django.utils.functional import Promise
 from django.utils.encoding import force_unicode
+from django.contrib.redirects.models import Redirect
 
 # lfs imports
 import lfs.catalog.utils
@@ -16,21 +18,27 @@ from lfs.caching.utils import lfs_get_object_or_404
 from lfs.core.models import Shop
 from lfs.catalog.models import Category
 
-class LazyEncoder(simplejson.JSONEncoder):
-    """Encodes django's lazy i18n strings.
+def l10n_float(string):
+    """Takes a country specfic decimal value as string and returns a float.
     """
-    def default(self, obj):
-        if isinstance(obj, Promise):
-            return force_unicode(obj)
-        return obj
+
+    # TODO: Implement a proper transformation with babel or similar
+    if settings.LANGUAGE_CODE == "de":
+        string = string.replace(",", ".")
+
+    try:
+        return float(string)
+    except ValueError:
+        return 0.0
 
 def get_default_shop():
-    """Returns the default shop. At the moment this the shop with id == 1.
+    """Returns the default shop.
     """
     return lfs_get_object_or_404(Shop, pk=1)
 
 def lfs_quote(string, encoding="utf-8"):
-    """Encodes string to encoding before quoting.
+    """Encodes passed string to passed encoding before quoting with
+    urllib.quote().
     """
     return urllib.quote(string.encode(encoding))
 
@@ -45,7 +53,7 @@ def import_module(module):
     return module
 
 def set_message_to(response, msg):
-    """Sets cookie to given response with given message.
+    """Sets message cookie with passed message to passed response.
     """
     # We just keep the message two seconds.
     max_age = 2
@@ -55,10 +63,10 @@ def set_message_to(response, msg):
 
     response.set_cookie("message", lfs_quote(msg), max_age=max_age, expires=expires)
     return response
-    
+
 def set_message_cookie(url, msg):
-    """Creates response object with given url and adds message cookie with passed
-    message.
+    """Returns a HttpResponseRedirect object with passed url and set cookie
+    ``message`` with passed message.
     """
     # We just keep the message two seconds.
     max_age = 2
@@ -71,8 +79,18 @@ def set_message_cookie(url, msg):
 
     return response
 
+def render_to_ajax_response(html=[], message=None):
+    """Encodes given html and message to JSON and returns a HTTP response.
+    """
+    result = simplejson.dumps(
+        { "message" : message, "html" : html }, cls = LazyEncoder)
+
+    return HttpResponse(result)
+
 def get_current_categories(request, object):
-    """Returns all current categories based on given request
+    """Returns all current categories based on given request. Current
+    categories are the current selected category and all parent categories of
+    it.
     """
     if object and object.content_type == "category":
         parents = object.get_parents()
@@ -89,39 +107,67 @@ def get_current_categories(request, object):
 
     return current_categories
 
-def render_to_ajax_response(html=[], message=None):
-    """Encodes given html and message to JSON and returns a HTTP response.
+def get_redirect_for(path):
+    """Returns redirect path for the passed path.
     """
-    result = simplejson.dumps(
-        { "message" : message, "html" : html }, cls = LazyEncoder)
+    try:
+        redirect = Redirect.objects.get(
+            site = settings.SITE_ID, old_path = path)
+    except Redirect.DoesNotExist:
+        return ""
+    else:
+        return redirect.new_path
+
+def set_redirect_for(old_path, new_path):
+    """Sets redirect path for the passed path.
+    """
+    try:
+        redirect = Redirect.objects.get(site = settings.SITE_ID, old_path=old_path)
+        redirect.new_path=new_path
+        redirect.save()
+    except Redirect.DoesNotExist:
+        redirect = Redirect.objects.create(
+            site_id=settings.SITE_ID, old_path=old_path, new_path=new_path)
+
+def remove_redirect_for(path):
+    """Removes the redirect path for given path.
+    """
+    try:
+        redirect = Redirect.objects.get(site = settings.SITE_ID, old_path=path)
+    except Redirect.DoesNotExist:
+        return False
+    else:
+        redirect.delete()
+        return True
 
-    return HttpResponse(result)
-    
 def set_category_levels():
-    """Creates category levels based on the position in hierarchy.
+    """Sets the category levels based on the position in hierarchy.
     """
     for category in Category.objects.all():
         category.level = len(category.get_parents()) + 1
         category.save()
 
 def get_start_day(date):
-    """Takes a string such as "2009-07-23" and returns a range of this day.
+    """Takes a string such as ``2009-07-23`` and returns datetime object of
+    this day.
     """
     year, month, day = date.split("-")
     start = datetime.datetime(int(year), int(month), int(day))
     return start
 
 def get_end_day(date):
-    """
+    """Takes a string such as ``2009-07-23`` and returns a datetime object with
+    last valid second of this day: 23:59:59.
     """
     year, month, day = date.split("-")
-    end = datetime.datetime(int(year), int(month), int(day))    
+    end = datetime.datetime(int(year), int(month), int(day))
     end = end + datetime.timedelta(1) - datetime.timedelta(microseconds=1)
-    
+
     return end
-  
+
 def getLOL(objects, objects_per_row=3):
-    """Returns a list of list of given objects.
+    """Returns a list of list of the passed objects with passed objects per
+    row.
     """
     result = []
     row = []
@@ -135,7 +181,15 @@ def getLOL(objects, objects_per_row=3):
         result.append(row)
 
     return result
-    
+
+class LazyEncoder(simplejson.JSONEncoder):
+    """Encodes django's lazy i18n strings.
+    """
+    def default(self, obj):
+        if isinstance(obj, Promise):
+            return force_unicode(obj)
+        return obj
+
 class CategoryTree(object):
     """Represents a category tree.
     """
