@@ -19,13 +19,14 @@ from django.utils.translation import ugettext_lazy as _
 
 # lfs imports
 import lfs.core.utils
+import lfs.core.signals
 import lfs.order.settings
 from lfs.caching.utils import lfs_get_object_or_404
 from lfs.core.utils import LazyEncoder
 from lfs.mail import utils as mail_utils
 from lfs.order.models import Order
 
-@permission_required("manage_shop", login_url="/login/")
+@permission_required("core.manage_shop", login_url="/login/")
 def manage_orders(request, template_name="manage/order/manage_orders.html"):
     """Dispatches to the first order or the order overview.
     """
@@ -37,7 +38,7 @@ def manage_orders(request, template_name="manage/order/manage_orders.html"):
         return HttpResponseRedirect(
             reverse("lfs_manage_order", kwargs={"order_id" : order.id}))
 
-@permission_required("manage_shop", login_url="/login/")
+@permission_required("core.manage_shop", login_url="/login/")
 def orders_view(request, template_name="manage/order/orders.html"):
     """Main view to display an order overview.
     """
@@ -45,6 +46,7 @@ def orders_view(request, template_name="manage/order/orders.html"):
         "orders_inline" : orders_inline(request, as_string=True),
     }))
 
+@permission_required("core.manage_shop", login_url="/login/")
 def orders_inline(request, as_string=False, template_name="manage/order/orders_inline.html"):
     """Displays the orders. This is factored out in order to reload it via
     ajax request.
@@ -90,6 +92,7 @@ def orders_inline(request, as_string=False, template_name="manage/order/orders_i
 
         return HttpResponse(result)
 
+@permission_required("core.manage_shop", login_url="/login/")
 def set_order_filters(request):
     """Sets order filters given by passed request.
     """
@@ -139,6 +142,7 @@ def set_order_filters(request):
 
     return HttpResponse(result)
 
+@permission_required("core.manage_shop", login_url="/login/")
 def set_order_filters_date(request):
     """Sets the date filter by given short cut link
     """
@@ -169,6 +173,7 @@ def set_order_filters_date(request):
 
     return HttpResponse(result)
 
+@permission_required("core.manage_shop", login_url="/login/")
 def reset_order_filters(request):
     """resets order filter.
     """
@@ -193,7 +198,7 @@ def reset_order_filters(request):
 
     return HttpResponse(result)
 
-@permission_required("manage_shop", login_url="/login/")
+@permission_required("core.manage_shop", login_url="/login/")
 def order_view(request, order_id, template_name="manage/order/order.html"):
     """Displays order with provided order id.
     """
@@ -231,7 +236,7 @@ def order_inline(request, order_id, as_string=False, template_name="manage/order
     else:
         return HttpResponse(result)
 
-@permission_required("manage_shop", login_url="/login/")
+@permission_required("core.manage_shop", login_url="/login/")
 def selectable_orders_inline(request, as_string=False,
     template_name="manage/order/selectable_orders_inline.html"):
     """Displays the selectable orders for the order view. (Used to switch
@@ -264,7 +269,7 @@ def selectable_orders_inline(request, as_string=False,
         return HttpResponse(result)
 
 # Actions
-@permission_required("manage_shop", login_url="/login/")
+@permission_required("core.manage_shop", login_url="/login/")
 def delete_order(request, order_id):
     """Deletes order with provided order id.
     """
@@ -279,7 +284,7 @@ def delete_order(request, order_id):
 
     return HttpResponseRedirect(url)
 
-@permission_required("manage_shop", login_url="/login/")
+@permission_required("core.manage_shop", login_url="/login/")
 def send_order(request, order_id):
     """Sends order with passed order id to the customer of this order.
     """
@@ -291,6 +296,7 @@ def send_order(request, order_id):
         msg = _(u"Order has been sent."),
     )
 
+@permission_required("core.manage_shop", login_url="/login/")
 def change_order_state(request):
     """Changes the state of an order, given by request post variables.
     """
@@ -298,9 +304,16 @@ def change_order_state(request):
     state_id = request.POST.get("new-state")
     order = get_object_or_404(Order, pk=order_id)
 
-    order.state = state_id
-    order.state_modified = datetime.now()
-    order.save()
+    try:
+        order.state = int(state_id)
+    except ValueError:
+        pass
+    else:
+        order.state_modified = datetime.now()
+        order.save()
+
+    if order.state == lfs.order.settings.SENT:
+        lfs.core.signals.order_sent.send({"order" : order, "request" : request})
 
     msg = _(u"State has been changed")
 
@@ -349,4 +362,4 @@ def _get_filtered_orders(order_filters):
 
     orders = orders.filter(created__range=(s, e))
 
-    return orders
\ No newline at end of file
+    return orders
